import json

from viaduct.core import *
from viaduct.utils import *


class ISA(CoreModule):
    def __init__(self, username, password, reality, browserPath, headless=False, timeout=2):
        super().__init__(username, password, TradingType.ISA,
                         reality, headless, browserPath, timeout)

    # Gets min and max buy and sell value in £, max sell quantity is number of shares
    # Not sure what sellThreshold is
    # The sell parameters only appear for UK stocks for some reason
    def getMinMax(self, code):
        return self.get("https://live.trading212.com/rest/v1/equity/value-order/min-max?instrumentCode=" + code, cookies=self.cookiePayload)

    # Takes instrument code, returns maxBuy and maxSell in shares for the account and max buy and sell that is technically
    # possible on the exchange, also has minTrade and if suspended
    def getSettings(self, code):
        return self.post("https://live.trading212.com/rest/v2/account/instruments/settings", cookies=self.cookiePayload, payload=[code])

    # Gets chart data fior a given ticker code
    def getChartData(self, ticker, chartPeriod, size, includeFake=False):
        payload = {
            "candles": [
                {
                    "ticker": ticker,
                    "period": chartPeriod,
                    "size": size,
                    "includeFake": includeFake
                }
            ]
        }
        return post("https://live.trading212.com/charting/v2/batch", cookies=self.cookiePayload, payload=payload)
    
    # Gets the account performance graph data
    def getPortfolioPerformance(self, historyPeriod):
        return get("https://live.trading212.com/rest/v2/portfolio?period=" + historyPeriod, cookies=self.cookiePayload)


"""

Standard REST request handler that doesn't need any account specific details or Selenium session cookie extraction

"""

import logging

from viaduct.utils import *


class Public:
    def __init__(self, loadSymbols=False):
        if (loadSymbols):
            self.saveAllInstruments()
        else:
            self.loadedInstruments = False

    # Get all instruments on Trading212!
    def getAllInstruments(self):
        return get("https://live.trading212.com/rest/v2/instruments/")

    # Gets company ISINs and tickers for graphs etc
    def getTickers(self):
        return get("https://live.trading212.com/rest/companies")

    # Saves all instruments in a dictionary useful for symbol lookup
    def saveAllInstruments(self):
        logging.info("Loading all instruments, this will take a while...")
        self.instruments = self.getAllInstruments()
        # Don't think getting tickers is actually required
        #self.tickerISINs = self.getTickers()
        self.loadedInstruments = True

    # Search the instrument dictionary
    def instrumentSearch(self, searchUsing, searchKey, resultUsing):
        if (not self.loadedInstruments):
            self.saveAllInstruments()
        for i in self.instruments:
            try:
                if (i[searchUsing] == searchKey):
                    return i[resultUsing]
            except:
                pass

    # Gets a symbols name
    def getName(self, isin="", prettyName="", code="", id=""):
        if (isin != ""):
            return self.instrumentSearch("isin", isin, "name")
        if (prettyName != ""):
            return self.instrumentSearch("prettyName", prettyName, "name")
        if (code != ""):
            return self.instrumentSearch("code", code, "name")
        if (id != ""):
            return self.instrumentSearch("id", id, "name")
        else:
            raise Exception(
                "ISIN, prettyName, code, or id is required to find name")

    # Gets a symbols prettyName
    def getPrettyName(self, isin="", name="", code="", id=""):
        if (isin != ""):
            return self.instrumentSearch("isin", isin, "prettyName")
        if (name != ""):
            return self.instrumentSearch("name", name, "prettyName")
        if (code != ""):
            return self.instrumentSearch("code", code, "prettyName")
        if (id != ""):
            return self.instrumentSearch("id", id, "prettyName")
        else:
            raise Exception(
                "ISIN, name, code, or id is required to find prettyName")

    # Gets a symbols code
    def getCode(self, isin="", name="", prettyName="", id=""):
        if (isin != ""):
            return self.instrumentSearch("isin", isin, "code")
        if (name != ""):
            return self.instrumentSearch("name", name, "code")
        if (prettyName != ""):
            return self.instrumentSearch("prettyName", prettyName, "code")
        if (id != ""):
            return self.instrumentSearch("id", id, "code")
        else:
            raise Exception(
                "ISIN, name, prettyName, or id is required to find code")

    # Gets a symbols ISIN
    def getISIN(self, name="", prettyName="", code="", id=""):
        if (name != ""):
            return self.instrumentSearch("name", name, "isin")
        if (prettyName != ""):
            return self.instrumentSearch("prettyName", prettyName, "isin")
        if (code != ""):
            return self.instrumentSearch("code", code, "isin")
        if (id != ""):
            return self.instrumentSearch("id", id, "isin")
        else:
            raise Exception(
                "Name, prettyName, code, or id is required to find ISIN")

    # Gets a symbols ID
    def getID(self, isin="", name="", prettyName="", code=""):
        if (isin != ""):
            return self.instrumentSearch("isin", isin, "id")
        if (name != ""):
            return self.instrumentSearch("name", name, "id")
        if (prettyName != ""):
            return self.instrumentSearch("prettyName", prettyName, "id")
        if (code != ""):
            return self.instrumentSearch("code", code, "id")
        else:
            raise Exception(
                "ISIN, name, prettyName, or code is required to find id")

    # Get instrument details position code from the secret API
    def getInstrument(self, code):
        return get("https://live.trading212.com/rest/v2/instruments/" + code)

    # Get instrument details by ISIN
    # If language is not available, Trading212 seems to return English
    def getFundamentals(self, isin, langCode="en"):
        return get("https://live.trading212.com/rest/companies/fundamentals?languageCode=" + langCode + "&isin=" + isin)

    # Gets chart data for a particular ticker
    # When getting chart data, the ticker returned is the code not what is on the stock exchange!
    def getChartData(self, code, chartPeriod, size, includeFake=False):
        payload = {
            "candles": [
                {
                    "ticker": code,
                    "period": "ONE_DAY",
                    "size": size,
                    "includeFake": includeFake
                }
            ]
        }
        return post(url="https://live.trading212.com/charting/v2/batch", payload=payload)



"""

Utilities and basic framework for multiple features

"""


from time import sleep

import json
import requests

# Try clicking something until it works
def force_click(element, sleep_time=1):
    while True:
        try:
            element.click()
            return
        except:
            sleep(sleep_time)

def post(url, cookies="", payload=""):
    headers = {'Cookie': cookies, 'Connection': 'keep-alive',
               'Pragma': 'no-cache',
               'Cache-Control': 'no-cache',
               'Accept': 'application/json',
               'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.96 Safari/537.36',
               'Content-Type': 'application/json',
               'Sec-GPC': '1',
               'Origin': 'https://live.trading212.com',
               'Sec-Fetch-Site': 'same-origin',
               'Sec-Fetch-Mode': 'cors',
               'Sec-Fetch-Dest': 'empty',
               'Referer': 'https://live.trading212.com/beta',
               'Accept-Language': 'en-GB,en-US;q=0.9,en;q=0.8'}
    
    response = requests.post(url, headers=headers,
                             data=json.dumps(payload))

    # response.headers is the response headers as x is the response
    # response.status code is number kind of response code

    # print(response.headers)

    if response.status_code == 400:
        raise Exception("Error 400 we made a bad requst")
    elif response.status_code == 404:
        raise Exception("Error 404 url not found")
    elif response.status_code == 401:
        raise Exception("Error 401 we are not authorised on Trading212")
    elif response.status_code == 500:
        raise Exception(
            "Error 500 Trading212 experienced an internal server error")
    elif response.status_code == 200:
        try:
            return json.loads(response.text)
        except:
            raise Exception(
                "No response body was received from the server")
    else:
        raise Exception("Error " + str(response.status_code) +
                        " encountered, unknown error")


def get(url, cookies="", payload=""):
    headers = {'Cookie': cookies, 'Connection': 'keep-alive',
               'Pragma': 'no-cache',
               'Cache-Control': 'no-cache',
               'Accept': 'application/json',
               'X-Trader-SeqId': '25',
               'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/88.0.4324.96 Safari/537.36',
               'Content-Type': 'application/json',
               'Sec-GPC': '1',
               'Origin': 'https://live.trading212.com',
               'Sec-Fetch-Site': 'same-origin',
               'Sec-Fetch-Mode': 'cors',
               'Sec-Fetch-Dest': 'empty',
               'Referer': 'https://live.trading212.com/beta',
               'Accept-Language': 'en-GB,en-US;q=0.9,en;q=0.8'}
    
    response = requests.get(url, headers=headers, data=json.dumps(payload))

    # response.headers is the response headers as x is the response
    # response.status code is number kind of response code
    
    # print(response.headers)

    if response.status_code == 400:
        raise Exception("Error 400 we made a bad requst")
    elif response.status_code == 404:
        raise Exception("Error 404 url not found")
    elif response.status_code == 401:
        raise Exception("Error 401 we are not authorised on Trading212")
    elif response.status_code == 200:
        try:
            return json.loads(response.text)
        except:
            raise Exception("No response body was received from the server")
    else:
        raise Exception("Error " + str(response.status_code) +
                        " encountered, unknown error")

# Enumerations


class TradingType:
    CFD = "CFD"
    Equity = "EQUITY"
    ISA = "ISA"


class Reality:
    Practice = "PRACTICE"
    Real = "REAL"


class BuyStockMethod:
    Shares = "Shares"
    Value = "Value"


class ChartPeriod:
    m1 = "ONE_MINUTE"
    m5 = "FIVE_MINUTES"
    m10 = "TEN_MINUTES"
    m15 = "FIFTEEN_MINUTES"
    m30 = "THIRTY_MINUTES"
    H1 = "ONE_HOUR"
    H4 = "FOUR_HOURS"
    D1 = "ONE_DAY"
    W1 = "ONE_WEEK"
    M1 = "ONE_MONTH"


class HistoryTimeframe:
    D1 = "LAST_DAY"
    W1 = "LAST_WEEK"
    M1 = "LAST_MONTH"
    M3 = "LAST_THREE_MONTHS"
    Y1 = "LAST_YEAR"
    ALL = "ALL"
